/*
 * PmodTMP3.c
 *
 *  Created on: Feb 26, 2018
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"

#include "PmodTMP3.h"

#define PMODTMP3_ADDRESS 0x90

I2C_HandleTypeDef pmodTmp3I2c;

void PmodTMP3_Config(int8_t alarm, int8_t hyst)
{
  // Configure the interrupt (PF15) for alarm.
  __HAL_RCC_GPIOF_CLK_ENABLE();
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Pin = GPIO_PIN_15;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 2, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

  pmodTmp3I2c.Instance = I2C2;
  pmodTmp3I2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  pmodTmp3I2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  pmodTmp3I2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  pmodTmp3I2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  pmodTmp3I2c.Init.OwnAddress1 = 0x01;

  // Set the I2C_TIMINGR register to fulfill the TCN75A timing requirements (max 400kHz).
  pmodTmp3I2c.Init.Timing = 0x10563046;

  HAL_I2C_Init(&pmodTmp3I2c);

  uint8_t tAlarm[2] = {alarm, 0};
  uint8_t tHyst[2] = {hyst, 0};

  HAL_I2C_Mem_Write(&pmodTmp3I2c, PMODTMP3_ADDRESS, 0x03, 1, tAlarm, 2, 100);
  HAL_I2C_Mem_Write(&pmodTmp3I2c, PMODTMP3_ADDRESS, 0x02, 1, tHyst, 2, 100);
}

void HAL_I2C_MspInit(I2C_HandleTypeDef *hi2c)
{
  __HAL_RCC_I2C2_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
  GPIO_InitStruct.Pin = GPIO_PIN_0 | GPIO_PIN_1;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
}

int16_t PmodTMP3_GetTemperature()
{
  // Read the current temperature value (2 bytes).
  uint8_t data[2] = {0};
  HAL_I2C_Mem_Read(&pmodTmp3I2c, PMODTMP3_ADDRESS, 0x00, 1, data, 2, 100);

  int16_t temperature = (data[0] << 8) | data[1];

  // The temperature value has 0.5°C resolution and is shifted left by 7 bits.
  // It is multiplied by 10 to keep the 0.5°C resolution of the result.
  temperature = (temperature * 10) >> 8;
  return temperature;
}
