/*
******************************************************************************
File:     main.c
Info:     Generated by Atollic TrueSTUDIO(R) 8.0.0   2017-10-28

The MIT License (MIT)
Copyright (c) 2009-2017 Atollic AB

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

******************************************************************************
*/

#include <PmodTMP3.h>
#include <string.h>

#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "system_clock.h"
#include "serial.h"
#include "led.h"

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  Serial_Config();
  Led_Config();

  // PmodTMP3 alarm.
  int16_t alarm = 28;
  int16_t hyst = 26;

  PmodTMP3_Config(alarm, hyst);

  // Check the temperature and turn on the LED if it exceeds the alarm threshold.
  int16_t temperature = PmodTMP3_GetTemperature();
  // The returned temperature is multiplied by 10.
  if (temperature > alarm * 10)
    Led_TurnOn(LED0);

  char textBuffer[128];   // Text buffer for serial writing.

  while(1) {
    HAL_Delay(1000);

    // Read the temperature value.
    temperature = PmodTMP3_GetTemperature();

    // Write the obtained value to the serial port.
    sprintf(textBuffer, "Temperature: %d\n", temperature);
    Serial_Write(textBuffer, strlen(textBuffer));
  }
}

/**
 * @brief This function handles the interrupts configured in the PmodTEMP3 driver.
 *        In the example there is only single interrupt source active so checking
 *        its source is not necessary.
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  // Change the LED state any time the interrupt is detected.
  Led_Change(LED0);
}
